-- TotalVector:  Electric field intensity vector emanating from the test charge and representing
-- the total effect of both fixed charges.


property ancestor


on new me, nSpr, lParams
  if (NOT lParams[ #oSelf ].objectP) then lParams[ #oSelf ] = me
  
  ancestor = (script "Vector").new( nSpr, lParams )
  
  me.makeSupplierList()
  return me
end new


-- add
--   Renders this vector as the sum of two component vectors.
-- params
--   oVector1 - [required] instance of class "Vector"
--   oVector2 - [required] instance of class "Vector"
--   ptStart  - [required] point from which the total vector should emanate

on add me, lParams
  oVector1 = lParams.getProp( #oVector1 )
  oVector2 = lParams.getProp( #oVector2 )
  ptStart  = lParams.getProp( #ptStart )
  
  -- Get the magnitude (in V/m) and direction (in radians) of the first vector.
  -- Break its magnitude down into X and Y components
  nMagnitude1  = oVector1.getMagnitudeVM()
  nDirection1  = oVector1.getDirectionRad()
  nMagnitudeX1 = nMagnitude1 * cos( nDirection1 ) 
  nMagnitudeY1 = nMagnitude1 * sin( nDirection1 ) 
  
  -- Do the same for the second vector.
  nMagnitude2  = oVector2.getMagnitudeVM()
  nDirection2  = oVector2.getDirectionRad()
  nMagnitudeX2 = nMagnitude2 * cos( nDirection2 ) 
  nMagnitudeY2 = nMagnitude2 * sin( nDirection2 ) 
  
  -- Compute the total vector's magnitude (in pixels) and direction (in degrees).
  nMagnitudeX     = float( nMagnitudeX1 + nMagnitudeX2 ) * ancestor.PIXELS_PER_VM
  nMagnitudeY     = float( nMagnitudeY1 + nMagnitudeY2 ) * ancestor.PIXELS_PER_VM
  nMagnitudeTotal = sqrt( power( nMagnitudeX, 2 ) + power( nMagnitudeY, 2 ) )
  nDirectionRad   = atan( nMagnitudeY / nMagnitudeX )
  nDirectionDeg   = nDirectionRad * (180/PI)
  
  if (nMagnitudeX < 0) then nDirectionDeg = nDirectionDeg - 180
  
  -- Draw the total vector's stem.
  me.setLoc(         ptStart ) 
  me.setWidth(       nMagnitudeTotal )
  me.setMagnitudeVM( nMagnitudeTotal / ancestor.PIXELS_PER_VM )
  me.setRotation(    nDirectionDeg )
  
  -- Draw the total vector's arrow head.
  nEndX     = ptStart[1] + nMagnitudeX
  nEndY     = ptStart[2] + nMagnitudeY
  me.getHead().setLoc( point( nEndX, nEndY ) )
  me.getHead().setRotation( nDirectionDeg ) 
end add