-- Charge:  Electrostatic charge with fixed position and magnitude, but possibly variable polarity


property ancestor         -- visible circular "forbidden zone" around the charge
property poConnector      -- line connecting this charge to the test charge
property poDot            -- center of the charge
property poVector         -- vector emanating from the test charge, resulting from this charge's effect upon it
property poRadiusReadout  -- numeric display of distance between charge's center and test charge
property pnPolarity       -- location of the center of this charge (doesn't change)
property pptCenter        -- location of the center of this charge (doesn't change)
property pnRadius         -- radius of the circular "forbidden zone" field around this charge
property MM_PER_PIXEL     -- millimeter : pixel scale


on new me, nSpr, lParams
  if (NOT lParams[ #oSelf ].objectP) then lParams[ #oSelf ] = me
  
  ancestor        = (script "SpriteObj").new( nSpr,      lParams )
  
  poConnector     = (script "SpriteObj").new( nSpr +  1, [:] )
  poDot           = (script "SpriteObj").new( nSpr +  2, [:] )
  poRadiusReadout = (script "Field").    new( nSpr +  3, [#nBorder: 1] )
  poVector        = (script "Vector").   new( nSpr +  8, [:] )
  
  MM_PER_PIXEL    = 0.24  -- 90 millimeters = 375 pixels
  
  me.makeSupplierList()
  return me
end new


-- renderEffectOn
--   Displays this charge's relationship to, and effect on, the test charge:  draws a
--   line connecting the two, displays a numeric readout of the distance between the
--   charges, and draws a vector showing this's charge's electric field intensity vector,
--   which emanates from the test charge.
-- params
--   oTestCharge - [required] 
-- notes
--   The two charge sprites are both offset 1 pixel from the corner of the background,
--   which acts as a constraint on the test charge.  The offsets prevent nDeltaX from 
--   equaling zero and making nTangent infinite.

on renderEffectOn me, oTestCharge
  ptTestCharge = oTestCharge.op( #loc )
  nAngleDeg    = me.angleTo( ptTestCharge )
  nDistancePix = distance( me.getCenter(), ptTestCharge )
  nDistanceMM  = nDistancePix * MM_PER_PIXEL
  sDistanceMM  = string( integer( nDistanceMM ) )
  
  poConnector.setRotation( nAngleDeg )
  poConnector.setWidth( nDistancePix )  
  poRadiusReadout.op( #setString, sDistanceMM )
  
  lParams = [#nAngleDeg   : nAngleDeg,        \
             #nDistanceMM : nDistanceMM,      \
             #nPolarity   : me.getPolarity(), \
             #ptTestCharge: ptTestCharge      ]
  poVector.draw( lParams )
end renderEffectOn


-- angleTo
--   Returns the angle from the center of this charge to an arbitrary point
-- params
--   ptPoint - [required]
-- returns 
--   angle (degrees) from this charge to the given point

on angleTo me, ptPoint
  nDeltaX   = float( ptPoint[1] - me.getCenter()[1] )
  nDeltaY   = float( ptPoint[2] - me.getCenter()[2] )
  nTangent  = nDeltaY / nDeltaX
  nAngleRad = atan( nTangent )
  nAngleDeg = nAngleRad * (180/PI)
  
  if (nDeltaX < 0) then nAngleDeg = nAngleDeg + 180
  
  return (nAngleDeg)
end angleTo


-- getCenter
--   Returns the location of the center of the charge.
-- note
--   Since the location of this charge never changes, saving its location as a property
--   may save a few CPU cycles.

on getCenter me
  if (pptCenter.voidP) then pptCenter = poDot.loc()
  return (pptCenter) 
end getCenter


-- getCenter
--   Returns the radius of the "forbidden zone" -- the circular area around this charge,
--   within which the user is not allowed to drag the test charge.  (Dragging the test
--   charge too close would make the vector unmanageably large.)
-- note
--   Since the radius of this charge's field never changes, saving its location as a
--   property may save a few CPU cycles.

on getFieldRadius me
  if (NOT pnRadius.integerP) then pnRadius = me.width() / 2
  return (pnRadius)
end getFieldRadius


on getPolarity me
  if (NOT pnPolarity.integerP) then pnPolarity = +1
  return (pnPolarity)
end getPolarity

on reversePolarity me
  pnPolarity = me.getPolarity()
  pnPolarity = -pnPolarity
  me.client().updateTestCharge()
end reversePolarity


on getVector me
  return (poVector)
end getVector
