-- Vector:  Electric field intensity vector emanating from the test charge.


property ancestor        -- stem (line)
property poHead          -- arrowhead
property poEFieldReadout -- numeric display of electric field magnitude
property PIXELS_PER_VM   -- pixel : V/m scale
property pnMagnitudeVM   -- field strength (vector magnitude), in V/m
property pnDirectionRad  -- vector direction, in radians


on new me, nSpr, lParams
  if (NOT lParams[ #oSelf ].objectP) then lParams[ #oSelf ] = me
  
  ancestor        = (script "SpriteObj").new( nSpr,      lParams )
  
  poHead          = (script "SpriteObj").new( nSpr +  1, [:] )
  poEFieldReadout = (script "Field").    new( nSpr +  2, [#nBorder: 1] )
  
  PIXELS_PER_VM = 0.0208  -- 200 V/m = 1 millimeter = 4.167 pixels
  
  me.makeSupplierList()
  return me
end new


-- draw 
-- 
-- params
--   nAngleDeg   - [required] angle between fixed charge and test charge (degrees)
--   nDistanceMM - [required] distance between fixed charge and test charge (millimeters)
--   nPolarity   - [required] -1 or +1
--   ptStart     - [required] position at which the vector's stem should start

on draw me, lParams
  nAngleDeg   = lParams.getProp( #nAngleDeg )
  nDistanceMM = lParams.getProp( #nDistanceMM )
  nPolarity   = lParams.getProp( #nPolarity )
  ptStart     = lParams.getProp( #ptTestCharge )
  
  if (nPolarity < 0) then nAngleDeg = nAngleDeg - 180
  
  -- Position the stem and turn it to the appropriate angle.
  me.setLoc( ptStart )
  me.setRotation( nAngleDeg )
  
  -- Calculate the magnitude of the field (in V/m), remember that number,
  -- and then convert it to pixels so that we can stretch the stem sprite
  -- the right amount.
  nMagnitudeVM = (9000000 / power( nDistanceMM, 2 ))  -- V/m
  me.setMagnitudeVM( nMagnitudeVM )
  me.setWidth( nMagnitudeVM * PIXELS_PER_VM )         -- pixels
  
  -- Position the arrow head and turn it to the appropriate angle.
  nAngleRad = nAngleDeg * (PI/180)
  nEndX     = ptStart[1] + cos( nAngleRad ) * nMagnitudeVM * PIXELS_PER_VM
  nEndY     = ptStart[2] + sin( nAngleRad ) * nMagnitudeVM * PIXELS_PER_VM
  poHead.setLoc( point( nEndX, nEndY ) )
  poHead.setRotation( nAngleDeg ) 
  
  -- Remember the vector's direction (in radians).
  me.setDirectionRad( nAngleRad )
end draw


on getMagnitudeVM me
  return (pnMagnitudeVM)
end getMagnitudeVM

on setMagnitudeVM me, nMagnitudeVM
  pnMagnitudeVM = nMagnitudeVM
  sMagnitudeVM  = string( integer( nMagnitudeVM )  )
  poEFieldReadout.op( #setString, sMagnitudeVM )
end setMagnitudeVM


on getDirectionRad me
  return (pnDirectionRad)
end getDirectionRad

on setDirectionRad me, nDirectionRad
  pnDirectionRad = nDirectionRad
end setDirectionRad


on getHead me
  return (poHead)
end getHead